@@ -35,13 +35,15 @@ from selve.util.errors import *
 from selve.util.protocol import ParameterType
 
 
-def _worker(selve: Selve):
+def _worker(selve: Selve, stop):
     # Infinite loop to collect all incoming data
     selve._LOGGER.debug("Reader started")
     try:
         while True:
             if not selve._pauseReader:
                 with selve._readLock:
+                    if not selve._serial.is_open:
+                        selve._serial.open()
                     if selve._serial.in_waiting > 0:
                         msg = ""
                         while True:
@@ -76,6 +78,10 @@ def _worker(selve: Selve):
                     time.sleep(0.5)
 
             time.sleep(0.01)
+            if stop():
+                selve._LOGGER.debug('Exiting worker loop...')
+                break
+        return True
     # serial port exceptions, all of these notify that we are in some
     # serious trouble
     except serial.SerialException:
@@ -91,6 +97,7 @@ class Selve:
         self._callbacks = set()
         self.lastLogEvent = None
         self.state = None
+        self._stopThread = False
 
         # Data from Duty Cycle Event
         self.utilization = 0
@@ -121,7 +128,7 @@ class Selve:
 
         self._setup()
         
-        self.readLoopTask = threading.Thread(target=_worker, args=(self, ))
+        self.readLoopTask = threading.Thread(target=_worker, args=(self, lambda: self._stopThread))
         self.readLoopTask.daemon = False
         self.readLoopTask.start()
 
@@ -172,17 +179,34 @@ class Selve:
                     xonxoff=False,
                     rtscts=False,
                     dsrdtr=False)
-                if self.pingGateway():
-                    self._port = p.name
-                    return
-                else:
-                    self._serial = None
-            except PortError:
+            except Exception as e:
+                self._LOGGER.error("Error at com port: " + str(e))
+                try:
+                    self._serial.close()
+                except:
+                    self._LOGGER.debug("Cannot close com port")
                 pass
+            if self.pingGateway():
+                self._port = p.name
+                return
+            else:
+                self._serial.close()
+                self._serial = None
         else:
             self._LOGGER.error("No gateway on comports found!")
             raise PortError
 
+    def stopGateway(self):
+        # wait for the rx/tx thread to end, these need to be gathered to
+        # collect all the exceptions
+        self._stopThread = True
+        self.readLoopTask.join()
+        # close the serial port, do the cleanup
+        if self._serial.is_open:
+            self._serial.close()
+        self._serial = None
+
+
     def register_callback(self, callback: Callable[[], None]) -> None:
         """Register callback, called when Roller changes state."""
         self._callbacks.add(callback)
@@ -203,14 +227,6 @@ class Selve:
                 time.sleep(0.5)
         except Exception as e:
             self._LOGGER.error("error communicating: " + str(e))
-        # self.writer.close()
-
-    # close the serial port, do the cleanup
-    def close(self):
-        # wait for the rx/tx thread to end, these need to be gathered to
-        # collect all the exceptions
-        self.readLoopTask.join(5)
-        self._serial.close()
 
     def processResponse(self, xmlstr):
         """Processes an XML String into a response object."""
@@ -222,6 +238,10 @@ class Selve:
         xmlstr = str(xmlstr).replace('<?xml version="1.0"? encoding="UTF-8">', '<?xml version="1.0" encoding="UTF-8"?>')
         try:
             res = untangle.parse(xmlstr)
+        except Exception as e:
+            self._LOGGER.error("Error in XML: " + str(e) + " : " + xmlstr)
+            return False
+        try:
             if not hasattr(res, 'methodResponse') and not hasattr(res, 'methodCall'):
                 self._LOGGER.error("Bad response format")
                 return None
@@ -232,6 +252,10 @@ class Selve:
                     response = self.create_response(res)
             else:
                 response = self.create_response_call(res)
+        except Exception as e:
+            self._LOGGER.error("Error in response creation: " + str(e) + " : " + xmlstr)
+            return False
+        try:
             # if it's a MethodResponse, it has not been sent by the gateway itself, so we can safely return it
             # otherwise it's an event, and we have to process it accordingly
             if isinstance(response, CommeoDeviceEventResponse) \
@@ -250,7 +274,7 @@ class Selve:
             return response
 
         except Exception as e:
-            self._LOGGER.error("Error in XML: " + str(e) + " : " + xmlstr)
+            self._LOGGER.error("Error in response processing: " + str(e) + " : " + xmlstr)
             return False
 
     def create_error(self, obj):
@@ -473,6 +497,12 @@ class Selve:
         return MethodResponse(methodName, flat_params_list)
 
     def executeCommand(self, command: Command):
+        if not self.readLoopTask.is_alive():
+            self._stopThread = False
+            self.readLoopTask = threading.Thread(target=_worker, args=(self, lambda: self._stopThread))
+            self.readLoopTask.daemon = False
+            self.readLoopTask.start()
+
         self.txQ.put_nowait(command)
 
 
@@ -519,7 +549,11 @@ class Selve:
                     
                     if isinstance(resp, ErrorResponse):
                         self._LOGGER.error(resp.message)
-                        raise GatewayError
+                        # retry
+                        try:
+                            return self._executeCommandSyncWithResponse(command)
+                        except GatewayError as e:
+                            raise e
                     else:
                         #time.sleep(0.5)
                         return resp
@@ -529,6 +563,9 @@ class Selve:
 
 
     def discover(self):
+
+        self._stopThread = True
+
         if self.gatewayReady():
             iveoIds: IveoGetIdsResponse = self.executeCommandSyncWithResponse(IveoGetIds())
             deviceIds: DeviceGetIdsResponse = self.executeCommandSyncWithResponse(DeviceGetIds())
@@ -626,6 +663,10 @@ class Selve:
                 device.sun2Analog = config.sun2Analog
                 device.sun3Analog = config.sun3Analog
                 self.addOrUpdateDevice(device, SelveTypes.SENSIM)
+        self._stopThread = False
+        self.readLoopTask = threading.Thread(target=_worker, args=(self, lambda: self._stopThread))
+        self.readLoopTask.daemon = False
+        self.readLoopTask.start()
         self.list_devices()
 
 
@@ -633,11 +674,11 @@ class Selve:
         for device in self.devices[SelveTypes.DEVICE.value]:
             self.updateCommeoDeviceValues(device.id)
         for sensor in self.devices[SelveTypes.SENSOR.value]:
-            pass
+            self.updateSensorValuesAsync(sensor.id)
         for senSim in self.devices[SelveTypes.SENSIM.value]:
-            pass
+            self.updateSenSimValuesAsync(senSim.id)
         for sender in self.devices[SelveTypes.SENDER.value]:
-            pass
+            self.updateSenderValuesAsync(sender.id)
 
 
 
@@ -767,6 +808,10 @@ class Selve:
             pass
         self._LOGGER.info("Gateway reset")
 
+    def setEvents(self, eventDevice = False, eventSensor = False, eventSender = False, eventLogging = False, eventDuty = False):
+        command = param.ParamSetEvent(eventDevice, eventSensor, eventSender, eventLogging, eventDuty)
+        self.executeCommand(command)
+
     def processEventResponse(self, response):
         if isinstance(response, CommeoDeviceEventResponse):
             # This is a commeo device response, Iveo does not generate events because it is a one way communication protocol
@@ -859,21 +904,21 @@ class Selve:
         
     def updateCommeoDeviceValuesFromResponse(self, id: int, response: DeviceGetValuesResponse):
         dev = self.getDevice(id, SelveTypes.DEVICE)
-        dev.name = response.name
-        dev.state = response.movementState
-        dev.value = response.value
-        dev.targetValue = response.targetValue
-        dev.unreachable = response.unreachable
-        dev.overload = response.overload
-        dev.obstructed = response.obstructed
-        dev.alarm = response.alarm
-        dev.lostSensor = response.lostSensor
-        dev.automaticMode = response.automaticMode
-        dev.gatewayNotLearned = response.gatewayNotLearned
-        dev.windAlarm = response.windAlarm
-        dev.rainAlarm = response.rainAlarm
-        dev.freezingAlarm = response.freezingAlarm
-        dev.dayMode = response.dayMode
+        dev.name = response.name if response.name else "None"
+        dev.state = response.movementState if response.movementState else MovementState.UNKOWN.value
+        dev.value = response.value if response.value else 0
+        dev.targetValue = response.targetValue if response.targetValue else 0
+        dev.unreachable = response.unreachable if response.unreachable else True
+        dev.overload = response.overload if response.overload else False
+        dev.obstructed = response.obstructed if response.obstructed else False
+        dev.alarm = response.alarm if response.alarm else False
+        dev.lostSensor = response.lostSensor if response.lostSensor else False
+        dev.automaticMode = response.automaticMode if response.automaticMode else False
+        dev.gatewayNotLearned = response.gatewayNotLearned if response.gatewayNotLearned else False
+        dev.windAlarm = response.windAlarm if response.windAlarm else False
+        dev.rainAlarm = response.rainAlarm if response.rainAlarm else False
+        dev.freezingAlarm = response.freezingAlarm if response.freezingAlarm else False
+        dev.dayMode = response.dayMode if response.dayMode else False
         self.addOrUpdateDevice(dev, SelveTypes.DEVICE)
         
     def setDeviceValue(self, id: int, value: int, type: SelveTypes):
@@ -975,4 +1020,17 @@ class Selve:
     def stopGroup(self, group: SelveGroup, type=DeviceCommandType.MANUAL):
         self.executeCommand(CommandStopGroup(group.id, type))
         for id in Util.b64bytes_to_bitlist(group.mask):
-            self.updateCommeoDeviceValuesAsync(id)
\ No newline at end of file
+            self.updateCommeoDeviceValuesAsync(id)
+
+
+    ### Sensor
+    def updateSensorValuesAsync(self, id: int):
+        self.executeCommand(SensorGetValues(id))
+
+    ### SenSim
+    def updateSenSimValuesAsync(self, id: int):
+        self.executeCommand(SenSimGetValues(id))
+
+    ### Sender
+    def updateSenderValuesAsync(self, id: int):
+        self.executeCommand(SenderGetValues(id))
\ No newline at end of file
